#!/usr/bin/env python

from getopt import getopt, GetoptError
from os import unlink
from subprocess import Popen, PIPE
from sys import argv
from tempfile import NamedTemporaryFile

# usage info
def usage():
    print 'Usage: %s [--left <left.jpg>] [--right <right.jpg>] [--cross <cross.jpg>] <image.mpo>' % argv[0]
    print '    utility for working with Nintendo 3DS mpo images'
    print '    extracts the left and right images and combines them into a cross-eye 3D image'
    print ''
    print '        --left       save the left image to this file'
    print '        --right      save the right image'
    print '        --cross      generate a cross eye image'
    print ''
    print '          [also takes -l, -r, and -c short options]'
    print ''
    print '    you must supply at least one of --left --right and --cross'
    print '    note: any existing images will be overwritten'
    print ''
    print 'for more info: http://lacklustre.net/'
    exit(1)

# unlink a file, ignoring file not found
def try_unlink(path):
    try:
        unlink(path)
    except OSError, e:
        # ignore file not found
        if e.errno != 2:
            raise e

optlist, args = None, None
try:
    optlist, args = getopt(
        argv[1:],
        'hl:r:c:',
        [ 'help', 'left=', 'right=', 'cross=']
    )
except GetoptError, e:
    print 'Error: %s\n' % e
    usage()

# defaults
left = None
right = None
cross = None

# handle the arguments
for o, a in optlist:
    if o in ('-h', '--help'):
        usage();
    elif o in ('-l', '--left'):
        left = a
    elif o in ('-r', '--right'):
        right = a
    elif o in ('-c', '--cross'):
        cross = a

# make sure they gave us exactly one image
if len(args) != 1:
    print 'Error: must supply exactly one mpo file\n'
    usage()

mpo_file = args[0]

if left is None and right is None and cross is None:
    print 'Error: must supply at least one of --left --right and --cross\n'
    usage()

#
# right image
#

# generate the right tempfile if necessary
right_file = None
if not right:
    right_file = NamedTemporaryFile(suffix='.jpg')
    right = right_file.name

try_unlink(right)

# generate right image
# n.b., stdout=PIPE makes the command run silently
Popen(['exiftool', '-trailer:all=', mpo_file, '-o', right], stdout=PIPE).wait()

#
# left image
#
left_file = None
if left:
    left_file = open(left, 'wb')
else:
    left_file = NamedTemporaryFile(suffix='.jpg')
    left = left_file.name

# generate left image
Popen(['exiftool', mpo_file, '-mpimage2', '-b'], stdout=left_file).wait()

#
# cross eye image
#
if cross:
    try_unlink(cross)
    Popen(['montage', left, right, '-geometry', '640x480+0+0', cross]).wait()

# get rid of the temp files
if right_file: right_file.close()
left_file.close() # if this is a temp file it will be deleted
